/*
 * (c) Copyright 2002 - 2004 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#ifndef _CONF_H_
#define _CONF_H_

#include <dirent.h>

#include "defs.h"
#include "almusvcu.h"
#include "apui_defs.h"
#include "sysmon.h"

/* BruteFIR include */
#include <bfmod.h>

/*
 * This is static configuration. After initialised by 'conf_parse_static', the
 * configuration will not change, and thus this struct must only be used for
 * read-only access.
 */
extern struct conf_static {
    int n_uimodules;
    struct ui_module_functions *uimodules[CONF_UIMODULES_MAX];
    char uimodules_path[PATH_MAX];
    int allocate_ram;
    char log_path[PATH_MAX];
    char storage_path[PATH_MAX];
    int storage_size;
    int log_rotate_size;
    int master_input_device;
    int n_rates;
    int rates[FILTERPRG_RATES_MAX];
    char brutefir_path[PATH_MAX];
    char brutefir_modules_path[PATH_MAX];
    char cpu_temp_proc_path[PATH_MAX];
    char motherboard_temp_proc_path[PATH_MAX];
    bool_t run_as_daemon;
    bool_t suspend_in_runtime;
    bool_t allow_poll_mode;
    int min_device_blocksize;
    char suspend_command[PATH_MAX];
    struct {
        int n_devices;
	char device[CONF_DEVICES_MAX][PATH_MAX];
	char device_params[CONF_DEVICES_MAX][PATH_MAX];
        int total_open_channels;
	int open_channels[CONF_DEVICES_MAX];
	int channel_index[CONF_DEVICES_MAX][VCU_MAX_CHANNELS];
	int n_sample_formats;
	int sample_formats[CONF_SAMPLE_FORMATS_MAX];
    } input;
    struct {
        int n_devices;
	char device[CONF_DEVICES_MAX][PATH_MAX];
	char device_params[CONF_DEVICES_MAX][PATH_MAX];
        int total_open_channels;
	int open_channels[CONF_DEVICES_MAX];
	int channel_index[CONF_DEVICES_MAX][VCU_MAX_CHANNELS];
	int n_sample_formats;
	int sample_formats[CONF_SAMPLE_FORMATS_MAX];
    } output;
} conf_static;

struct stage {
    double azimut;
    double elevation;
    double width;
    double height;
};

struct speakerpos {
    bool_t active;
    double azimut;
    double elevation;
    double distance;
};

struct fprogref {
    prgid_t id;
    char path[PATH_MAX];
};

/* FIXME: in/out array index on the input/output pairs found in the struct
   below */

extern struct conf_user {
    struct {
	double master;
        double group[VCU_MAX_CHANNEL_GROUPS];
        bool_t mute[2][VCU_CH_WP_COUNT][VCU_MAX_CHANNELS];
        bool_t group_mute[VCU_MAX_CHANNEL_GROUPS];
	bool_t master_temp_mute;
    } volume;
    struct {
	int n_rates;
	int rates[VCU_VALID_RATES_COUNT];
	int suggested_rate;
	int output_sample_format;
	bool_t use_soft_clip;
	bool_t apply_dither;
        bool_t watch_clock;
        bool_t double_precision;
    } signal;
    struct {
        int count[2][VCU_CH_WP_COUNT];
        uint32_t map[2][VCU_CH_COUNT][VCU_MAX_CHANNELS][VCU_MAX_CHANNELS/32];
        int group_count;
        struct vcu_chgrp group[VCU_MAX_CHANNEL_GROUPS];
	struct speakerpos speakerpos[VCU_MAX_CHANNELS];
        int stage_count;
        struct stage stage[VCU_MAX_STAGES];
        bool_t distance_compensate;
	struct {
            double volume[2][VCU_CH_WP_COUNT][VCU_MAX_CHANNELS];
	    double delay[2][VCU_CH_WP_COUNT][VCU_MAX_CHANNELS];
	} trim;
        int mode_count;
        struct vcu_chmode mode[VCU_MAX_CHANNEL_MODES];
        int active_mode;
    } channel;
    struct {
        int eqprg_count;
	struct vcu_eqprg eqprg[VCU_MAX_EQ_PROGRAMS];
	int n_loaded_programs;
	prgid_t loaded_programs[CONF_MAX_EQ_PROGRAMS];
        int mode_count;
        struct vcu_eqmode mode[VCU_MAX_EQ_MODES];
        int active_mode;
    } eq;
    struct {
	bool_t use_xtalk_cancel[VCU_MAX_CHANNELS/2];
	prgid_t current[VCU_MAX_CHANNELS/2];
	int n_loaded_programs;
	prgid_t loaded_programs[CONF_MAX_XTC_PROGRAMS];
    } ambiopole;
    struct {
	struct basic_sysinfo benchmark_sysinfo;
        double max_benchmark_load;
        double max_benchmark_length;
        bool_t iodelay_manually_set;
	int block_sizes[VCU_VALID_RATES_COUNT];
	int n_blocks[VCU_VALID_RATES_COUNT];
	prgid_t current;
	int n_loaded_programs;
	prgid_t loaded_programs[BFIR_MAX_REVERB_PROGRAMS];
        int response_mode;
        int decorrelate_mode;
        bool_t force_delay_diff;
        bool_t force_decay;
        bool_t mix_with_dry;
        double dry_volume;
        double bformat_directivity;
        int source_layout;
    } reverb;
    struct {
        int input_format;
    } ambisonics;
    struct {
        bool_t powersave;
        bool_t debug;
        bool_t load_user_config;
        bool_t load_revprg_config;
        bool_t load_static_config;
        int n_fprogrefs;
        struct fprogref *fprogrefs[CONF_MAX_FPROGREFS];
    } ctl;
} cu;

struct conf_revprg {
    prgid_t id;
    double volume;
    double delay;
    double scale_length;
    int response_mode;
    int decorrelate_mode;
    bool_t force_delay_diff;
    bool_t force_decay;
    bool_t mix_with_dry;
    double dry_volume;
    double bformat_directivity;
    double xyz_balance[3];
};

void
conf_load_static(const char path[]);

void
conf_load_user(void);

void
conf_save_user(void);

void
conf_load_revprg(struct conf_revprg *revprg,
		 const struct filterprg *fprg);

void
conf_save_revprg(const struct conf_revprg *revprg);

void
conf_create_pidfile(void);

#endif
