/*
 * (c) Copyright 2001 - 2002 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#ifndef _GRAMMAR_H_
#define _GRAMMAR_H_

#include <stdio.h>
#include <inttypes.h>
#include <math.h>

#include "defs.h"
#include "almusvcu.h"
#include "bit.h"

#define EOS     VCU_LEX_EOS /* end of statement (;) */
#define COMMA   VCU_LEX_COMMA /* , */
#define SLASH   VCU_LEX_SLASH /* / */
#define LBRACE  VCU_LEX_LBRACE /* { */
#define RBRACE  VCU_LEX_RBRACE /* } */

#define BOOLEAN VCU_LEXVAL_BOOLEAN
#define REAL    VCU_LEXVAL_REAL
#define STRING  VCU_LEXVAL_STRING
#define FIELD   VCU_LEXVAL_FIELD

#define INVALID_INPUT -2

extern union vcu_lexval yylval;
extern FILE *yyin;
extern int yylineno;

/* A few useful macros for the parser */
#define GET_TOKEN(token) {                                                     \
    int t;                                                                     \
    if ((t = yylex()) != token) {                                              \
	yy_perror_tok(token, t);                                               \
        return false;                                                          \
    }                                                                          \
}
#define GET_INTEGER() {                                                        \
    int t;                                                                     \
    if ((t = yylex()) != REAL) {                                               \
	yy_perror_tok(REAL, t);                                                \
	return false;                                                          \
    }                                                                          \
    if (rint(yylval.real) != yylval.real) {                                    \
	yy_perror("expected integer, got floating point.");                    \
	return false;                                                          \
    }                                                                          \
}
#define TEST_AND_SET(bitpos)                                                   \
    if (bit_isset(isset, bitpos)) {                                            \
	yy_perror("field is already set.");                                    \
	return false;                                                          \
    }                                                                          \
    bit_set(isset, bitpos);

int
yylex(void);

void
yy_restart(const char filename[],
	   FILE *input_file,
           int loglevel);

void
yy_input_mode(bool_t set_early_eof,
	      int max_bytes);

const char *
yy_tokname(int token);

void
yy_perror(const char format[],
	  ...);

void
yy_pwarning(const char format[],
            ...);

void
yy_perror_tok(int expected_token,
	      int got_token);

#endif
