/*
 * (c) Copyright 2001, 2005 - 2006 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
  
#include "defs.h"
#include "grammar.h"

union vcu_lexval yylval;

static bool_t is_early_eof = false;
static int early_eof_bytes_left = 0;

#define YY_INPUT(buf, result, max_size)                                        \
{                                                                              \
    if (is_early_eof) {                                                        \
	if (early_eof_bytes_left == 0) {                                       \
	    result = 0;                                                        \
	} else if (early_eof_bytes_left > max_size) {                          \
	    if (((result = fread(buf, 1, max_size, yyin)) == 0) &&             \
		ferror(yyin))                                                  \
	    {                                                                  \
		YY_FATAL_ERROR( "input in flex scanner failed" );              \
	    }                                                                  \
	    early_eof_bytes_left -= result;                                    \
	} else {                                                               \
	    if (((result = fread(buf, 1, early_eof_bytes_left, yyin)) == 0) && \
		ferror(yyin))                                                  \
	    {                                                                  \
		YY_FATAL_ERROR( "input in flex scanner failed" );              \
	    }                                                                  \
	    early_eof_bytes_left -= result;                                    \
	}                                                                      \
    } else {                                                                   \
	if (((result = fread(buf, 1, max_size, yyin)) == 0) && ferror(yyin)) { \
	    YY_FATAL_ERROR( "input in flex scanner failed" );                  \
	}                                                                      \
    }                                                                          \
}

void
yy_input_mode(bool_t set_early_eof,
	      int max_bytes)
{
    yylineno = 1;
    is_early_eof = set_early_eof;
    early_eof_bytes_left = max_bytes;
}
%}

%option nounput
%option noyywrap
%option yylineno

DIGIT [0-9]
ANUM [A-Za-z_0-9]

%%

"\""("\\\""|[^\"])*"\"" {
    int n, slen;
    char numstr[4];
    slen = strlen(yytext) - 1;
    for (n = 1; n < slen - 1; n++) {
        if (yytext[n] == '\\') {
            memmove(&yytext[n], &yytext[n+1], slen - n);
            slen--;
            switch ((int)yytext[n]) {
            case '0' ... '9':
                numstr[0] = yytext[n];
                numstr[1] = '\0';
                if (yytext[n+1] >= '0' && yytext[n+1] <= '9') {
                    numstr[1] = yytext[n+1];
                    numstr[2] = '\0';
                    slen--;
                    if (yytext[n+2] >= '0' && yytext[n+2] <= '9') {
                        numstr[2] = yytext[n+2];
                        numstr[3] = '\0';
                        slen--;
                        memmove(&yytext[n+1], &yytext[n+3], slen - n);
                    } else {
                        memmove(&yytext[n+1], &yytext[n+2], slen - n);
                    }
                }
                yytext[n] = atoi(numstr);
                break;
            case 'a': yytext[n] = '\a'; break;
            case 'b': yytext[n] = '\b'; break;
            case 't': yytext[n] = '\t'; break;
            case 'n': yytext[n] = '\n'; break;
            case 'v': yytext[n] = '\v'; break;
            case 'f': yytext[n] = '\f'; break;
            case 'r': yytext[n] = '\r'; break;
            }
        }
    }
    yytext[slen] = '\0';
    yylval.string = &yytext[1];
    return STRING;   
}

","       { return COMMA; }
"/"       { return SLASH; }
";"       { return EOS; }
"{"       { return LBRACE; }
"}"       { return RBRACE; }

"#"[^\n]*  /* remove one-line comments */

[ \t\n\r]+   /* remove whitespace and linebreaks */

"true" {
    yylval.boolean = true;
    return BOOLEAN;
}

"false" {
    yylval.boolean = false;
    return BOOLEAN;
}

{ANUM}+":" {
    yylval.field = yytext;
    yylval.field[strlen(yylval.field)-1] = '\0';
    return FIELD;
}

("+"|"-")?{DIGIT}*"."?{DIGIT}+("e"("+"|"-"){DIGIT}{2})? {
    yylval.real = atof(yytext);
    return REAL;
}

<<EOF>> {
    return EOF;
}

. {
    return INVALID_INPUT;
}

%%
