/*
 * This is a maximally equidistributed combined Tausworthe generator, stolen
 * from GNU Scientific Library (GSL). See GSL documentation for further
 * information.
 *
 * Generates numbers between 0x0 - 0xFFFFFFFF
 */
#ifndef _TAUSRAND_H_
#define _TAUSRAND_H_

static inline uint32_t
tausrand(uint32_t state[3])
{
#define TAUSWORTHE(s,a,b,c,d) ((s & c) << d) ^ (((s <<a) ^ s) >> b)  
    state[0] = TAUSWORTHE(state[0], 13, 19, (uint32_t)4294967294U, 12);
    state[1] = TAUSWORTHE(state[1], 2, 25, (uint32_t)4294967288U, 4);
    state[2] = TAUSWORTHE(state[2], 3, 11, (uint32_t)4294967280U, 17);
#undef TAUSWORTHE
    return (state[0] ^ state[1] ^ state[2]);
}

static inline void
tausinit(uint32_t state[3],
	 uint32_t seed)
{
    /* default seed is 1 */
    if (seed == 0) {
        seed = 1; 
    }
#define LCG(n) ((69069 * n) & 0xFFFFFFFFU)
    state[0] = LCG(seed);
    state[1] = LCG(state[1]);
    state[2] = LCG(state[2]);
#undef LCG
    /* "warm it up" */
    tausrand(state);
    tausrand(state);
    tausrand(state);
    tausrand(state);
    tausrand(state);
    tausrand(state);
}

#endif
